"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const ts = require("typescript");
const visitor_generate_1 = require("./visitor-generate");
function transformNode(node, visitorContext) {
    if (ts.isCallExpression(node)) {
        const signature = visitorContext.checker.getResolvedSignature(node);
        if (signature !== undefined
            && signature.declaration !== undefined
            && path.resolve(signature.declaration.getSourceFile().fileName) === path.resolve(path.join(__dirname, '..', '..', 'index.d.ts'))
            && node.typeArguments !== undefined
            && node.typeArguments.length === 1) {
            const typeArgument = node.typeArguments[0];
            const type = visitorContext.checker.getTypeFromTypeNode(typeArgument);
            const accessor = ts.createIdentifier('object');
            const arrowFunction = createArrowFunction(accessor, type, visitorContext);
            const typeFlags = type.getFlags();
            const isTypeObject = typeFlags === ts.TypeFlags.Object;
            const isDateType = isTypeObject && type.symbol && type.symbol.name === "Date";
            const isTypeUndefined = typeFlags === ts.TypeFlags.Undefined;
            const isTypeNull = typeFlags === ts.TypeFlags.Null;
            const shouldGenerateFromScratch = node.arguments[0] === undefined;
            if ((isTypeObject && !isDateType) || (shouldGenerateFromScratch && !isTypeUndefined && !isTypeNull)) {
                return ts.createCall(arrowFunction, undefined, node.arguments);
            }
            else if (isTypeNull) {
                return ts.createNull();
            }
            else if (isDateType) {
                return ts.createNew(ts.createIdentifier("Date"), undefined, [node.arguments[0]]);
            }
            else {
                return node.arguments[0];
            }
        }
    }
    return node;
}
exports.transformNode = transformNode;
function createArrowFunction(_accessor, type, visitorContext) {
    const expression = visitor_generate_1.visitType(type, visitorContext);
    return ts.createArrowFunction(undefined, undefined, [
        ts.createParameter(undefined, undefined, undefined, ts.createIdentifier('prop'), ts.createToken(ts.SyntaxKind.QuestionToken), ts.createKeywordTypeNode(ts.SyntaxKind.AnyKeyword))
    ], undefined, undefined, ts.createBlock([
        ts.createReturn(ts.createCall(ts.createPropertyAccess(ts.createIdentifier('Object'), ts.createIdentifier('assign')), undefined, [
            expression,
            ts.createIdentifier('prop')
        ]))
    ]));
}
//# sourceMappingURL=transform-node.js.map