"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const path = require("path");
const visitor_1 = require("./visitor");
const compiler_1 = require("./compiler");
function transformer(program, options) {
    if (options === undefined) {
        throw new Error('Options are required.');
    }
    if (options.outFile === undefined) {
        throw new Error('Option outFile is required.');
    }
    const outFile = options.outFile;
    const visitorContext = {
        program,
        checker: program.getTypeChecker(),
        typeDefinitions: {},
        typeDefinitionsAccessor: ts.createIdentifier('symbols'),
        typeCheckFunctionAccessor: ts.createIdentifier('generate'),
        typeCheckFunctionAccessorTopLevel: null,
        typeArgumentsStack: []
    };
    let remaining = program.getRootFileNames().length;
    return (context) => {
        return (file) => {
            const result = transformNodeAndChildren(file, program, context, visitorContext);
            if (--remaining === 0) {
                const sourceFile = ts.createSourceFile(outFile, '', ts.ScriptTarget.ES2016, undefined, ts.ScriptKind.JS);
                const newSourceFile = ts.updateSourceFileNode(sourceFile, compiler_1.compile(visitorContext));
                const printer = ts.createPrinter({
                    newLine: ts.NewLineKind.LineFeed
                });
                const result = printer.printFile(newSourceFile);
                ts.sys.writeFile(newSourceFile.fileName, result);
            }
            return result;
        };
    };
}
exports.default = transformer;
function transformNodeAndChildren(node, program, context, visitorContext) {
    return ts.visitEachChild(transformNode(node, visitorContext), (childNode) => transformNodeAndChildren(childNode, program, context, visitorContext), context);
}
function transformNode(node, visitorContext) {
    if (ts.isCallExpression(node)) {
        const signature = visitorContext.checker.getResolvedSignature(node);
        if (signature !== undefined
            && signature.declaration !== undefined
            && path.resolve(signature.declaration.getSourceFile().fileName) === path.resolve(path.join(__dirname, '..', '..', 'index.d.ts'))
            && node.arguments.length <= 1
            && node.typeArguments !== undefined
            && node.typeArguments.length === 1) {
            const typeArgument = node.typeArguments[0];
            const accessor = node.arguments[0] || ts.createNull();
            return visitor_1.visitTypeNode(typeArgument, accessor, Object.assign({}, visitorContext, { typeCheckFunctionAccessorTopLevel: node.expression }));
        }
    }
    return node;
}
//# sourceMappingURL=transformer.js.map